"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var common_1 = require('@angular/common');
var http_1 = require('@angular/http');
var router_1 = require('@angular/router');
var index_1 = require('./../services/index');
var index_2 = require('./../interfaces/index');
var index_3 = require('./../models/index');
var index_4 = require('./../pipes/index');
var WrapperPipe = (function () {
    function WrapperPipe() {
        this.registeredPipes = new Map();
        if (!this.registeredPipes.get("date"))
            this.registeredPipes.set("date", new common_1.DatePipe('en-US'));
        if (!this.registeredPipes.get("msDate"))
            this.registeredPipes.set("msDate", new index_4.MSDatePipe());
    }
    WrapperPipe.prototype.transform = function (value, args) {
        if (!args || args.length === 0 || !args[0] || args[0].length === 0)
            return value;
        else {
            var pipes = args[0].split(',');
            for (var ix = 0; ix < pipes.length; ix++)
                if (this.registeredPipes.get(pipes[ix]))
                    value = this.registeredPipes.get(pipes[ix]).transform(value, null);
            return value;
        }
    };
    WrapperPipe = __decorate([
        core_1.Pipe({ name: 'pipeWrapper' }), 
        __metadata('design:paramtypes', [])
    ], WrapperPipe);
    return WrapperPipe;
}());
exports.WrapperPipe = WrapperPipe;
var EntityListControl = (function () {
    function EntityListControl(logger, gds, http, router, entityService, zone, applicationRef) {
        var _this = this;
        this.logger = logger;
        this.http = http;
        this.router = router;
        this.entityService = entityService;
        this.zone = zone;
        this.applicationRef = applicationRef;
        this.dataLoadOnInit = true;
        this.editLink = "";
        this.filters = null;
        this.showEntityList = new Array();
        this.showEntity = new index_3.BaseEntity();
        this.EntityDataStructure = index_3.BaseEntity;
        this.fields = new Array();
        this.rerenderRequired = true;
        this.cd = 0;
        var that = this;
        setInterval(function () {
            if (_this.cd != _this.showEntityList.length) {
                _this.cd = _this.showEntityList.length;
                that.applicationRef.tick();
            }
        }, 250);
        logger.log("Entity list component initiated!");
        this.myIDComponent = ++EntityListControl.idComponent;
        if (!EntityListControl.destroyedOnes)
            EntityListControl.destroyedOnes = new Array();
    }
    Object.defineProperty(EntityListControl.prototype, "entityType", {
        set: function (EntityType) {
            this.showEntity = new EntityType();
            this.EntityDataStructure = EntityType;
            this.fields = [].concat(this.showEntity.browseFields);
            this.registerDataService();
        },
        enumerable: true,
        configurable: true
    });
    EntityListControl.prototype.setFilters = function (newFilters) {
        this.filters = newFilters;
        var that = this;
        if (this.filters && this.filters.length > 0 && this.showEntity) {
            this.entityService.loadData(this.EntityDataStructure, this.filters).then(function (res) { that.showEntityList = res.records; });
        }
    };
    EntityListControl.prototype.ngOnDestroy = function () {
        if (this.subscription) {
            EntityListControl.destroyedOnes.push(this.myIDComponent);
            this.subscription.unsubscribe();
            this.logger.log("ENTITYLIST destroyed: (ID: " + this.myIDComponent + ', filters: ' + ((this.filters) ? this.filters.length : 0) + ')');
        }
    };
    EntityListControl.prototype.triggerRefreshData = function () {
        try {
            this.entityService.reloadData(this.EntityDataStructure);
        }
        catch (e) {
            alert(e);
        }
    };
    EntityListControl.prototype.newEntity = function () {
        this.openDetail(this.showEntity.getNewInstance());
    };
    EntityListControl.prototype.openDetail = function (entity) {
        if (this.editLink.length > 0) {
            this.router.navigate([this.editLink, { id: entity.ID }]);
        }
        else {
            this.router.navigate([this.showEntity.getNameID() + '_DetailComponent', { id: entity.ID }]);
        }
    };
    EntityListControl.prototype.ngAfterViewInit = function () {
        this.logger.log('Entity list component AfterViewInit invoked.');
    };
    EntityListControl.prototype.ngOnInit = function () {
        this.logger.log('Entity list component OnInit invoked.');
        this.rerenderRequired = false;
    };
    EntityListControl.prototype.registerDataService = function () {
        var _this = this;
        if (this.showEntity) {
            var that = this;
            this.subscription = this.entityService.dataObserver.subscribe(function (updatedEntities) {
                if (!EntityListControl.destroyedOnes.find(function (x) { return x == _this.myIDComponent; })) {
                    that.logger.log("ENTITYLIST UPDATED DATA: EntityListID: " + that.myIDComponent);
                    if (!that.filters || that.filters.length === 0) {
                        that.showEntityList = that.entityService.getCurrentLibrary(that.EntityDataStructure);
                    }
                    else {
                        that.showEntityList = that.entityService.getCurrentLibrary(that.EntityDataStructure, that.filters);
                    }
                }
            });
            if (this.dataLoadOnInit) {
                try {
                    this.entityService.reloadData(this.EntityDataStructure);
                }
                catch (e) {
                    alert(e);
                }
                if (this.filters && this.filters.length > 0) {
                    this.entityService.loadData(this.EntityDataStructure, this.filters).then(function (res) { that.showEntityList = res.records; });
                }
                else {
                    this.showEntityList = this.entityService.getCurrentLibrary(this.EntityDataStructure);
                }
            }
        }
    };
    EntityListControl.idComponent = 0;
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], EntityListControl.prototype, "dataLoadOnInit", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object), 
        __metadata('design:paramtypes', [Object])
    ], EntityListControl.prototype, "entityType", null);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], EntityListControl.prototype, "editLink", void 0);
    EntityListControl = __decorate([
        core_1.Component({
            selector: 'entity-list',
            template: '<table class="table table-bordered table-striped mb-none dataTable no-footer"><tr><th *ngFor="let browseField of fields">{{browseField.Name}}</th><th>About</th></tr><tr *ngFor="let entityObj of showEntityList"><td *ngFor="let browseField of fields">{{entityObj[browseField.Name] | pipeWrapper:browseField.Pipe}}</td><td><button class="mb-xs mt-xs mr-xs btn btn-sm btn-info" (click)="openDetail(entityObj)">Detail</button></td></tr></table><button type="button" class="info" (click)="triggerRefreshData()">Refresh {{ showEntity.getEntityName() }}(s)</button> <button type="button" class="info" (click)="newEntity()">New {{ showEntity.getEntityName() }}</button>',
        }), 
        __metadata('design:paramtypes', [index_1.TestLogger, index_3.GlobalDataSharing, http_1.Http, router_1.Router, index_2.IEntityDataService, core_1.NgZone, core_1.ApplicationRef])
    ], EntityListControl);
    return EntityListControl;
}());
exports.EntityListControl = EntityListControl;
