"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var router_1 = require('@angular/router');
var index_1 = require('./../interfaces/index');
var index_2 = require('./../models/index');
var GridComponent = (function () {
    function GridComponent(elRef, router, entityService, cd) {
        this.elRef = elRef;
        this.router = router;
        this.entityService = entityService;
        this.cd = cd;
        this.editLink = "";
        this.detailsEnabled = true;
        this.filters = null;
        this.showEntityList = new Array();
        this.showEntity = null;
        this.entityDataStructure = index_2.BaseEntity;
        this.controlID = ++GridComponent.staticID;
    }
    Object.defineProperty(GridComponent.prototype, "entityType", {
        set: function (EntityType) {
            this.showEntity = this.entityService.getDummy(EntityType);
            this.entityDataStructure = EntityType;
            this.registerDataService();
        },
        enumerable: true,
        configurable: true
    });
    GridComponent.prototype.setFilters = function (newFilters) {
        this.filters = newFilters;
        var that = this;
        if (this.filters && this.filters.length > 0 && this.gridContainer && this.showEntity) {
            this.entityService.loadData(this.entityDataStructure, this.filters).then(function (res) {
                that.showEntityList = res.records;
                that.gridContainer.pqGrid('option', 'dataModel.data', that.showEntityList);
                that.gridContainer.pqGrid('refreshDataAndView');
            });
        }
    };
    GridComponent.prototype.openDetail = function (entity) {
        if (this.editLink.length > 0) {
            this.router.navigate([this.editLink, { id: entity.ID }]);
        }
        else {
            var url = 'comp/' + this.showEntity.getModuleName().toLowerCase() + '/' + this.showEntity.getEntityName().toLowerCase() + '/detail/' + entity.ID;
            this.router.navigateByUrl(url);
        }
    };
    GridComponent.prototype.registerDataService = function () {
        var _this = this;
        if (this.gridContainer && this.showEntity) {
            if (this.columns.length > 0) {
                this.gridColumns = this.columns.map(function (x) {
                    return {
                        title: x.Name,
                        dataIndx: x.Name,
                        dataType: x.DataType,
                        width: 150
                    };
                });
            }
            else {
                this.gridColumns = this.showEntity.browseFields.map(function (x) {
                    return {
                        title: x.Name,
                        dataIndx: x.Name,
                        dataType: x.DataType,
                        width: 150
                    };
                });
            }
            if (this.detailsEnabled) {
                this.gridColumns.push({
                    title: "", editable: false, minWidth: 50, sortable: false, render: function (ui) {
                        return "<a class='view_details'>Details</a>";
                    }
                });
            }
            var that = this;
            var toolbarActions = [];
            if (this.detailsEnabled) {
                toolbarActions.push({
                    type: 'button', icon: 'ui-icon-plus', label: 'New', listeners: [
                        {
                            "click": function (evt, ui) {
                                that.openDetail(that.showEntity.getNewInstance());
                            }
                        }
                    ]
                });
            }
            this.gridContainer.pqGrid({
                colModel: this.gridColumns,
                dataModel: { data: this.showEntityList },
                toolbar: {
                    items: toolbarActions
                },
                refresh: function () {
                    var grid = jQuery(this);
                    if (!grid)
                        return;
                    grid.find("a.view_details")
                        .unbind("click")
                        .bind("click", function (evt) {
                        var tr = jQuery(this).closest("tr"), rowIndx = grid.pqGrid("getRowIndx", { $tr: tr }).rowIndx;
                        that.openDetail(grid.pqGrid('option', 'dataModel.data')[rowIndx]);
                        return false;
                    });
                }
            });
            this.gridContainer.pqGrid("option", "scrollModel.autoFit", true);
            this.cd.detectChanges();
            this.subscription = this.entityService.dataObserver.subscribe(function (updatedEntities) {
                if (!_this.filters || _this.filters.length === 0) {
                    _this.showEntityList = _this.entityService.getCurrentLibrary(_this.entityDataStructure);
                }
                else {
                    _this.showEntityList = _this.entityService.getCurrentLibrary(_this.entityDataStructure, _this.filters);
                }
                _this.gridContainer.pqGrid('option', 'dataModel.data', _this.showEntityList);
                _this.gridContainer.pqGrid('refreshDataAndView');
            });
            try {
                this.entityService.reloadData(this.entityDataStructure);
            }
            catch (e) {
                alert(e);
            }
            if (this.filters && this.filters.length > 0) {
                this.entityService.loadData(this.entityDataStructure, this.filters).then(function (res) {
                    _this.showEntityList = res.records;
                    _this.gridContainer.pqGrid('option', 'dataModel.data', _this.showEntityList);
                    _this.gridContainer.pqGrid('refreshDataAndView');
                });
            }
            else {
                this.showEntityList = this.entityService.getCurrentLibrary(this.entityDataStructure);
                this.gridContainer.pqGrid('option', 'dataModel.data', this.showEntityList);
                this.gridContainer.pqGrid('refreshDataAndView');
            }
        }
    };
    GridComponent.prototype.ngOnDestroy = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    };
    GridComponent.prototype.ngAfterViewInit = function () {
        jQuery(this.elRef.nativeElement).find("#gridContainer").attr('id', 'gridContainer_' + this.controlID);
        this.gridContainer = jQuery(jQuery(this.elRef.nativeElement).find('#gridContainer_' + this.controlID));
        this.registerDataService();
    };
    GridComponent.staticID = 1;
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object), 
        __metadata('design:paramtypes', [Object])
    ], GridComponent.prototype, "entityType", null);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], GridComponent.prototype, "editLink", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], GridComponent.prototype, "detailsEnabled", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Array)
    ], GridComponent.prototype, "columns", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Array)
    ], GridComponent.prototype, "filters", void 0);
    GridComponent = __decorate([
        core_1.Component({
            selector: 'base-grid',
            encapsulation: core_1.ViewEncapsulation.Emulated,
            template: "<div id=\"gridContainer\"></div>"
        }), 
        __metadata('design:paramtypes', [core_1.ElementRef, router_1.Router, index_1.IEntityDataService, core_1.ChangeDetectorRef])
    ], GridComponent);
    return GridComponent;
}());
exports.GridComponent = GridComponent;
