"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var router_1 = require('@angular/router');
var http_1 = require('@angular/http');
var Rx_1 = require('rxjs/Rx');
var app_settings_1 = require('./../app/app.settings');
var index_1 = require('./../interfaces/index');
var index_2 = require('./../services/index');
var index_3 = require('./../models/index');
var LoginService = (function () {
    function LoginService(http, entityService, router) {
        var _this = this;
        this.http = http;
        this.entityService = entityService;
        this.router = router;
        this.userLoggedIn = false;
        this.loggedAs = 'Guest';
        var cookie = document.cookie;
        var that = this;
        this.UserLoginInfoObservable.subscribe(function (loginInfo) {
            that.userLoggedIn = loginInfo.UserLoggedIn;
            that.loggedAs = loginInfo.LoggedAs;
        });
        var data = {};
        try {
            data = JSON.parse(cookie.match(app_settings_1.AppSettings.COOKIES_FORMAT)[1]);
        }
        catch (_) { }
        if (data.UserName) {
            var that = this;
            that.userLoggedIn = false;
            this.http.get(app_settings_1.AppSettings.PING_ENDPOINT_URL).subscribe(function (_) {
                console.log('Ping OK: ' + _);
                that.UserLoginInfoObservable.next({ UserLoggedIn: true, LoggedAs: data.UserName, Message: "User login data valid: " + data.UserName, Success: true });
            }, function (_) {
                console.log('Ping failure. Clearing user info from cookie.');
                document.cookie = 'rhetosFormsAuth=' + JSON.stringify({ UserName: '' });
                that.UserLoginInfoObservable.next({ UserLoggedIn: false, LoggedAs: "Guest", Message: "Invalid login user data: " + data.UserName, Success: false });
            }, function () { });
        }
        else {
            this.entityService.getCurrentPrincipal().then(function (principalData) {
                if (principalData.records.length > 0) {
                    console.log('User if granted to enter');
                    var currentUser = principalData.records[0];
                    document.cookie = 'rhetosFormsAuth=' + JSON.stringify({ UserName: currentUser.Name });
                    that.UserLoginInfoObservable.next({ UserLoggedIn: true, LoggedAs: currentUser.Name, Message: "User has valid authentication cookie for " + currentUser.Name, Success: true });
                }
                else {
                    _this.entityService.userHasPermission(index_3.Claim, 'Read').then(function (result) {
                        var user = "Administrator (override)";
                        document.cookie = 'rhetosFormsAuth=' + JSON.stringify({ UserName: user });
                        that.UserLoginInfoObservable.next({ UserLoggedIn: true, LoggedAs: user, Message: "User has valid authentication cookie for " + user, Success: true });
                    });
                }
            }).catch(function (_) {
                console.log("User is not authorized.");
                that.UserLoginInfoObservable.next({ UserLoggedIn: false, LoggedAs: "Guest", Message: null, Success: false });
            });
        }
    }
    Object.defineProperty(LoginService.prototype, "UserLoginInfoObservable", {
        get: function () { return null; },
        enumerable: true,
        configurable: true
    });
    LoginService.prototype.login = function (params) {
        var headers = new http_1.Headers({ 'Content-Type': 'application/json' });
        var options = new http_1.RequestOptions({ headers: headers });
        var that = this;
        this.http.post(app_settings_1.AppSettings.LOGIN_ENDPOINT + '/login', JSON.stringify(params), options).subscribe(function (res) {
            if (res.text() == "true") {
                console.log('Login success: ' + res.text());
                if (params.PersistCookie) {
                    document.cookie = 'rhetosFormsAuth=' + JSON.stringify({ UserName: params.UserName });
                }
                that.UserLoginInfoObservable.next({ UserLoggedIn: true, LoggedAs: params.UserName, Message: "Successful login: " + params.UserName, Success: true });
            }
            else {
                console.log('Login failure: ' + res.text());
                that.UserLoginInfoObservable.next({ UserLoggedIn: false, LoggedAs: that.loggedAs, Message: "Unsuccessful login: " + res.text(), Success: false });
            }
        }, function (res) {
            console.log('Login failure: ' + res.text());
            that.UserLoginInfoObservable.next({ UserLoggedIn: false, LoggedAs: that.loggedAs, Message: "Unsuccessful login: " + res.text(), Success: false });
        }, function () { });
    };
    LoginService.prototype.logout = function (returnUrl) {
        var that = this;
        this.http.post(app_settings_1.AppSettings.LOGIN_ENDPOINT + '/logout', null).subscribe(function (res) {
            if (res.text() == "true") {
                console.log('Logout success: ' + res.text());
                document.cookie = 'rhetosFormsAuth=' + JSON.stringify({ UserName: '' });
                that.UserLoginInfoObservable.next({ UserLoggedIn: false, LoggedAs: "Guest", Message: "Successful Logout: Guest", Success: true });
                if (returnUrl) {
                    that.router.navigate([returnUrl]);
                }
            }
            else {
                console.log('Logout failure: ' + res.text());
                that.UserLoginInfoObservable.next({ UserLoggedIn: that.userLoggedIn, LoggedAs: that.loggedAs, Message: "Unsuccessful Logout: " + res.text(), Success: false });
            }
        }, function (res) {
            console.log('Logout failure: ' + res.text());
            that.UserLoginInfoObservable.next({ UserLoggedIn: false, LoggedAs: that.loggedAs, Message: "Unsuccessful Logout: " + res.text(), Success: false });
        }, function () { });
    };
    LoginService.prototype.register = function (params) {
        var headers = new http_1.Headers({ 'Content-Type': 'application/json' });
        var options = new http_1.RequestOptions({ headers: headers });
        var that = this;
        return this.http.post(app_settings_1.AppSettings.LOGIN_ENDPOINT + '/register', JSON.stringify(params), options).toPromise().then(function (res) {
            if (res.text() == "true") {
                console.log('Register request success: ' + res.text());
                return "true";
            }
            else {
                console.log('Register request failure: ' + res.text());
            }
        }, function (res) {
            console.log('Register request failure: ' + res.text());
            return res.text();
        });
    };
    __decorate([
        index_2.GloballyShared(new Rx_1.Subject()), 
        __metadata('design:type', Rx_1.Subject)
    ], LoginService.prototype, "UserLoginInfoObservable", null);
    LoginService = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [http_1.Http, index_1.IEntityDataService, router_1.Router])
    ], LoginService);
    return LoginService;
}());
exports.LoginService = LoginService;
